/******************************************************************************
  ezht - Copyright (C) 2004 Tradcrafts
  This file is part of the ezht.
                                                                       
  ezht is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by the 
  Free Software Foundation;   either version 2 of the License,  or (at your 
  option) any later version.

  ezht is distributed in the hope that it will be useful, but 
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; see the file COPYING.  If not, write to the Free Software 
  Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
******************************************************************************/

#include "reader.h"
#include "stack.h"
#include "error.h"

#include <string.h>
#include <deque>
#include <map>
#include <string>


typedef std::map< std::string , std::deque<std::string> > stack_t;
static stack_t stacks;

typedef std::map< std::string , void (*)(const char*) > checkstack_t;
static checkstack_t stack_checkers;

static void charset_checker(const char *charset)
{
  if( strcasecmp(charset,"EUC-JP")==0 ||
      strcasecmp(charset,"Shift_JIS")==0 ){
    return;
  }
      
  error("stack `charset' doesn't allowed the value `%s'",charset);
}

static void filename_rule_checker(const char *rule)
{
  if( strcmp(rule,"line")==0 ||
      strcmp(rule,"serial")==0 ||
      strcmp(rule,"section")==0 
      ){
    return;
  }
      
error("stack `rule' doesn't allowed the value `%s'",rule);
}


void stack_init()
{
  stacks[STACKNAME_LANG];

  stacks[STACKNAME_CHARSET];
  stack_checkers[STACKNAME_CHARSET] = charset_checker;

  stacks[STACKNAME_DOCNAME];

  stacks[STACKNAME_HT_HEAD];

  stacks[STACKNAME_BODY_HEADER];

  stacks[STACKNAME_BODY_FOOTER];

  stacks[STACKNAME_BODY_ATTRIBUTES];

  stacks[STACKNAME_INDEXFILE_INTRODUCTION];

  stacks[STACKNAME_SECTION_BAR_COLOR];

  stacks[STACKNAME_SECTION_MARK];

  stacks[STACKNAME_NUMBER_SEPARATOR];

  stacks[STACKNAME_TEXT_INDEX];

  stacks[STACKNAME_TEXT_RELATION];

  stacks[STACKNAME_TEXT_SECTION_DISJOINED];

  stacks[STACKNAME_ANCHOR_INDEX];

  stacks[STACKNAME_ANCHOR_UP];

  stacks[STACKNAME_ANCHOR_PREV];

  stacks[STACKNAME_ANCHOR_NEXT];

  stacks[STACKNAME_ANCHOR_BACK];

  stacks[STACKNAME_INDEXFILE];

  stacks[STACKNAME_FILENAME_PREFIX];

  stacks[STACKNAME_FILENAME_SUFFIX];

  stacks[STACKNAME_FILENAME_RULE];
  stack_checkers[STACKNAME_FILENAME_RULE] = filename_rule_checker;
  
  stacks[STACKNAME_INCLUDE_FILTER];

  stacks[STACKNAME_INCLUDE_HEADER];

  stacks[STACKNAME_INCLUDE_FOOTER];

  stacks[STACKNAME_TAB_SPACES];

}

void stack_push(const char *stackname, const char *body)
{
  stack_t::iterator p = stacks.find(stackname);
  if( p == stacks.end() ){
    error("stack `%s' doesn't exists",stackname);
    return ; // Not found.
  }

  p->second.push_front( body );
}

void stack_pop(const char *stackname)
{
  stack_t::iterator p = stacks.find(stackname);
  if( p != stacks.end() ){
    if( p->second.size() == 0 ){
      error("stack `%s' is already empty",stackname); return;
    }
    p->second.pop_front();
  }else{
    error("stack `%s' doesn't exists",stackname); return;
  }
}

const char *stack_peep(const char *elementname,const char *default_body)
{
  stack_t::iterator p = stacks.find(elementname);
  if( p != stacks.end() ){
    if( p->second.size() > 0 )
      return p->second[0].c_str();
    else
      return default_body;
  }else{
    error("internal error! %s",elementname);
    return NULL;
  }
}


static void check_stack_value(const char *stackname,const char *value)
{
  checkstack_t::iterator checker = stack_checkers.find(stackname);
  if( checker != stack_checkers.end() )
    (*checker->second)(value);
}

void cmd_push(std::vector<std::string> &args)
{
  if( args.size() > 3 ){
    error("@push: too many parameters");
    return;
  }
  
  if( args.size()==1 ){
    error("@push: element not specified");
    return;
  }
  
  if( args.size()==3 ){
    check_stack_value(args[1].c_str(),args[2].c_str());
    stack_push(args[1].c_str() ,args[2].c_str());
  }else if( args.size() == 2 ){
    
    std::string body;
    const char *line;
    while( NULL != (line=getline(true)) ){
      if( 0 == strcmp(line,"@end") )
	break;
      body += line;
    }
    if( line == NULL ){
      error("@push: unexpected end of file"); return;
    }
    check_stack_value(args[1].c_str(),body.c_str());
    stack_push(args[1].c_str() ,body.c_str()); 
  }
  
}

void cmd_pop(std::vector<std::string> &args)
{
  if( args.size() == 1 ){
    error("@pop: no stack specified"); return; 
  }
  
  int end = args.size();
  for(int i=1; i<end; i++)
    stack_pop(args[i].c_str());
}

void cmd_autopop(std::list<std::string> &autopop_list, std::vector<std::string> &args)
{
  if( args.size() == 1 ){
    warn("@autopop: no stack specified");
    return;
  }
  
  int n = (int)args.size();
  for(int i=1; i<n; i++){
    if( stacks.end() != stacks.find(args[i]) )
      autopop_list.push_back(args[i]);
    else
      error("@autopop: unknown stack `%s'",args[i].c_str()); 
  }
}
